/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */



import GLib from 'gi://GLib';
import Gio from 'gi://Gio';
import GObject from 'gi://GObject';
import Clutter from 'gi://Clutter';
import St from 'gi://St';

import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as Config from 'resource:///org/gnome/shell/misc/config.js';


const Indicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {    
        constructor(settings) {
            super(0.0, _('System Software Info'));            
            this._settings = settings;
            let _label = null;
            this._label = new St.Label({
                y_align: Clutter.ActorAlign.CENTER,         
                text: "Loading...",
                style_class: 'panel-button', 
                track_hover: false
            });
            this.add_child(this._label);            
            this._label.text = this._get_label(
                this._settings.get_boolean('show-os'), 
                this._settings.get_enum('os-style'), 
                this._settings.get_boolean('show-kernel'), 
                this._settings.get_enum('kernel-style'), 
                this._settings.get_boolean('show-shell'),
                this._settings.get_enum('shell-style'), 
                this._settings.get_enum('separator-style'), 
                this._settings.get_int('separator-width')
            )
        }
        
        _get_label(show_os, os_style, show_kernel, kernel_style, show_shell, shell_style, separator_style, separator_width) {    
            let separator_string = this._get_separator_string(separator_style, separator_width);
            let os_string = "";
            let kernel_string = "";
            let shell_string = "";
            if (!show_os && !show_kernel && !show_shell) {
                return "No info to show.";
            }
            if (show_os) {
                os_string = this._get_os_string(os_style);
            }
            if (show_kernel) {    
                if (show_os) {
                    kernel_string += separator_string;
                }
                kernel_string += this._get_kernel_string(kernel_style);        
            }
            if (show_shell) {     
                if (show_os || show_kernel) {
                    shell_string += separator_string;
                }
                shell_string += this._get_shell_string(shell_style);
            }
            return os_string + kernel_string + shell_string;
        }
        
        _get_os_string(os_style) {    
            let os_string = "";
            switch (os_style) {
                case 0:
                    os_string = GLib.get_os_info("PRETTY_NAME");
                    break;
                case 1: 
                    os_string = GLib.get_os_info("NAME") + ' ' + GLib.get_os_info("VERSION_ID"); 
                    break;
            }    
            return os_string;
        }
        
        _get_kernel_string(kernel_style) {
            const decoder = new TextDecoder();       
            let [res, out] = GLib.spawn_command_line_sync('uname -r'); //GLib.get_os_info doesn't provide kernel version
            let kernel_string = decoder.decode(out).split('-')[0];
            let prefix = "";
            switch (kernel_style) {
                case 2:
                    prefix = "Kernel";
                    break;
                case 1:
                    prefix = "Linux";
                    break;
                case 0:
                    prefix = "Linux Kernel";
                    break;
            }
            kernel_string = prefix + ' ' + kernel_string;  
            return kernel_string;
        }
        
        _get_shell_string(shell_style) {
            const shell_version = Config.PACKAGE_VERSION;
            let shell_string = "";
            switch (shell_style) {            
                case 0:
                    shell_string = "GNOME Shell";
                    break;
                case 1:
                    shell_string = "GNOME";
                    break;
            } 
            shell_string = shell_string + ' ' + shell_version;
            return shell_string;
        }
        
        _get_separator_string(separator_style, separator_width) {
            let separator_string = "";    
            if (separator_style != 7) {
                separator_string = this._get_spaces(separator_width);
            }
            switch (separator_style) {            
                case 0:            
                    separator_string += '•';
                    break;
                case 1:
                    separator_string += '–';        
                    break;
                case 2:
                    separator_string += '—';
                    break;
                case 3:            
                    separator_string += '|';
                    break;
                case 4:            
                    separator_string = ',';
                    break;
            }
            separator_string += this._get_spaces(separator_width);
            return separator_string;
        }
        
        _get_spaces(separator_width) {
            let spacing_string = '';
            for (let i = 0; i < separator_width; i++) {
                spacing_string += ' ';
            }
            return spacing_string;
        }
    }
);

export default class SystemSoftwareInfoExtension extends Extension {     
    _indicator;
    _settings;    
    _position;
    
    enable() {
        this._settings = this.getSettings();
        this._settings.connectObject('changed', this._init_indicator.bind(this), this);
        this._indicator = null;
        this._init_indicator();
    }
    
    _init_indicator() {
        if (this._indicator != null) {
            this._indicator.destroy();          
            this._indicator = null;
        }        
        this._indicator = new Indicator(this._settings);
        this._position = this._get_indicator_position();
        Main.panel.addToStatusArea(this.uuid, this._indicator, this._position["offset"], this._position["box"]);
    }
    
    _get_indicator_position() {
        let position_box_string = "";
        switch (this._settings.get_enum('position-box')) {            
            case 0:            
                position_box_string = 'left';
                break;
            case 1:
                position_box_string = 'center';        
                break;
            case 2:
                position_box_string = 'right';
                break;
        }
        return {
            "box": position_box_string,
            "offset": this._settings.get_int('position-offset')
        }
    }
    
    disable() {
        this._indicator.destroy();
        this._indicator = null;
    }
}
